#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <linux/soundcard.h>
#include <math.h>
#include <unistd.h>
#include <stdlib.h>
#include "rme96xx.h"

/*
  remctrl.c
  (C) 2000 Guenter Geiger <geiger@debian.org>
  HP20020201 - Heiko Purnhagen <purnhage@tnt.uni-hannover.de>
*/

/* # define DEVICE_NAME "/dev/mixer" */
# define DEVICE_NAME "/dev/mixer1"


void usage(void)
{
     fprintf(stderr,"usage: rmectrl [/dev/mixer<n>] [command [options]]\n\n");
     fprintf(stderr,"where command is one of:\n");
     fprintf(stderr,"  help                       show this help\n");
     fprintf(stderr,"  status                     show status bits\n");
     fprintf(stderr,"  control                    show control bits\n");
     fprintf(stderr,"  mix                        show mixer/offset status\n");
     fprintf(stderr,"  master <n>                 set sync master\n");
     fprintf(stderr,"  pro <n>                    set spdif out pro\n");
     fprintf(stderr,"  emphasis <n>               set spdif out emphasis\n");
     fprintf(stderr,"  dolby <n>                  set spdif out no audio\n");
     fprintf(stderr,"  optout <n>                 set spdif out optical\n");
     fprintf(stderr,"  wordclock <n>              set sync wordclock\n");
     fprintf(stderr,"  spdifin <n>                set spdif in (0=optical,1=coax,2=intern)\n");
     fprintf(stderr,"  syncref <n>                set sync source (0=ADAT1,1=ADAT2,2=ADAT3,3=SPDIF)\n");
     fprintf(stderr,"  adat1cd <n>                set ADAT1 on internal CD\n");
     fprintf(stderr,"  offset <devnr> <in> <out>  set dev (0..3) offset (0..25)\n");
     exit(-1);
}


int main(int argc, char* argv[])
{
     int cards;
     int ret;
     int i;
     double ft;
     int fd, fdwr;
     int param,orig;
     rme_status_t stat;
     rme_ctrl_t ctrl;
     char *device;
     int argidx;

     if (argc < 2)
	  usage();

     if (*argv[1]=='/') {
	  device = argv[1];
	  argidx = 2;
     }
     else {
	  device = DEVICE_NAME;
	  argidx = 1;
     }

     fprintf(stdout,"mixer device %s\n",device);
     if ((fd = open(device,O_RDONLY)) < 0) {
	  fprintf(stdout,"opening device failed\n");
	  exit(-1);
     }

     if ((fdwr = open(device,O_WRONLY)) < 0) {
	  fprintf(stdout,"opening device failed\n");
	  exit(-1);
     }

     if (argc < argidx+1)
	  usage();
	  
     if (!strcmp(argv[argidx],"help")) 
        usage();
     if (!strcmp(argv[argidx],"-h")) 
        usage();
     if (!strcmp(argv[argidx],"--help")) 
        usage();

     if (!strcmp(argv[argidx],"status")) {
	  ioctl(fd,SOUND_MIXER_PRIVATE2,&stat);
	  fprintf(stdout,"stat.irq %d\n",stat.irq);
	  fprintf(stdout,"stat.lockmask %d\n",stat.lockmask);
	  fprintf(stdout,"stat.sr48 %d\n",stat.sr48);
	  fprintf(stdout,"stat.wclock %d\n",stat.wclock);
	  fprintf(stdout,"stat.bufpoint %d\n",stat.bufpoint);
	  fprintf(stdout,"stat.syncmask %d\n",stat.syncmask);
	  fprintf(stdout,"stat.doublespeed %d\n",stat.doublespeed);
	  fprintf(stdout,"stat.tc_busy %d\n",stat.tc_busy);
	  fprintf(stdout,"stat.tc_out %d\n",stat.tc_out);
	  fprintf(stdout,"stat.crystalrate %d (0=64k 3=96k 4=88.2k 5=48k 6=44.1k 7=32k)\n",stat.crystalrate);
	  fprintf(stdout,"stat.spdif_error %d\n",stat.spdif_error);
	  fprintf(stdout,"stat.bufid %d\n",stat.bufid);
	  fprintf(stdout,"stat.tc_valid %d\n",stat.tc_valid);
	  exit (0);
     }
	 
     if (!strcmp(argv[argidx],"control")) {
	  ioctl(fd,SOUND_MIXER_PRIVATE3,&ctrl);
	  fprintf(stdout,"ctrl.start %d\n",ctrl.start);
	  fprintf(stdout,"ctrl.latency %d (0=64 .. 7=8192)\n",ctrl.latency);
	  fprintf(stdout,"ctrl.master %d\n",ctrl.master);
	  fprintf(stdout,"ctrl.ie %d\n",ctrl.ie);
	  fprintf(stdout,"ctrl.sr48 %d\n",ctrl.sr48);
	  fprintf(stdout,"ctrl.spare %d\n",ctrl.spare);
	  fprintf(stdout,"ctrl.doublespeed %d\n",ctrl.doublespeed);
	  fprintf(stdout,"ctrl.pro %d\n",ctrl.pro);
	  fprintf(stdout,"ctrl.emphasis %d\n",ctrl.emphasis);
	  fprintf(stdout,"ctrl.dolby %d\n",ctrl.dolby);
	  fprintf(stdout,"ctrl.opt_out %d\n",ctrl.opt_out);
	  fprintf(stdout,"ctrl.wordclock %d\n",ctrl.wordclock);
	  fprintf(stdout,"ctrl.spdif_in %d (0=optical,1=coax,2=intern)\n",ctrl.spdif_in);
	  fprintf(stdout,"ctrl.sync_ref %d (0=ADAT1,1=ADAT2,2=ADAT3,3=SPDIF)\n",ctrl.sync_ref);
	  fprintf(stdout,"ctrl.spdif_reset %d\n",ctrl.spdif_reset);
	  fprintf(stdout,"ctrl.spdif_select %d\n",ctrl.spdif_select);
	  fprintf(stdout,"ctrl.spdif_clock %d\n",ctrl.spdif_clock);
	  fprintf(stdout,"ctrl.spdif_write %d\n",ctrl.spdif_write);
	  fprintf(stdout,"ctrl.adat1_cd %d\n",ctrl.adat1_cd);
	  exit (0);
     }
     
     if (!strcmp(argv[argidx],"mix")) {
	  rme_mixer mix;
	  int i;
	  
	  for (i=0; i<4; i++) {
	       mix.devnr = i;
	       ioctl(fd,SOUND_MIXER_PRIVATE1,&mix);
	       if (mix.devnr == i) {
		    fprintf(stdout,"devnr %d\n",mix.devnr);
		    fprintf(stdout,"mix.i_offset %2d (0-25)\n",mix.i_offset);
		    fprintf(stdout,"mix.o_offset %2d (0-25)\n",mix.o_offset);
	       }
	  }
	  exit (0);
     }

/* the control flags */

     if (argc < argidx+2)
	  usage();
	  
     if (!strcmp(argv[argidx],"master")) {
	  int val = atoi(argv[argidx+1]);
	  ioctl(fd,SOUND_MIXER_PRIVATE3,&ctrl);
	  printf("master = %d\n",val);
	  ctrl.master = val;
	  ioctl(fdwr,SOUND_MIXER_PRIVATE3,&ctrl);
	  exit (0);
     }

     if (!strcmp(argv[argidx],"pro")) {
	  int val = atoi(argv[argidx+1]);
	  ioctl(fd,SOUND_MIXER_PRIVATE3,&ctrl);
	  printf("pro = %d\n",val);
	  ctrl.pro = val;
	  ioctl(fdwr,SOUND_MIXER_PRIVATE3,&ctrl);
	  exit (0);
     }

     if (!strcmp(argv[argidx],"emphasis")) {
	  int val = atoi(argv[argidx+1]);
	  ioctl(fd,SOUND_MIXER_PRIVATE3,&ctrl);
	  printf("emphasis = %d\n",val);
	  ctrl.emphasis = val;
	  ioctl(fdwr,SOUND_MIXER_PRIVATE3,&ctrl);
	  exit (0);
     }

     if (!strcmp(argv[argidx],"dolby")) {
	  int val = atoi(argv[argidx+1]);
	  ioctl(fd,SOUND_MIXER_PRIVATE3,&ctrl);
	  printf("dolby = %d\n",val);
	  ctrl.dolby = val;
	  ioctl(fdwr,SOUND_MIXER_PRIVATE3,&ctrl);
	  exit (0);
     }

     if (!strcmp(argv[argidx],"optout")) {
	  int val = atoi(argv[argidx+1]);
	  ioctl(fd,SOUND_MIXER_PRIVATE3,&ctrl);
	  printf("optout = %d\n",val);
	  ctrl.opt_out = val;
	  ioctl(fdwr,SOUND_MIXER_PRIVATE3,&ctrl);
	  exit (0);
     }

     if (!strcmp(argv[argidx],"wordclock")) {
	  int val = atoi(argv[argidx+1]);
	  ioctl(fd,SOUND_MIXER_PRIVATE3,&ctrl);
	  printf("wordclock = %d\n",val);
	  ctrl.wordclock = val;
	  ioctl(fdwr,SOUND_MIXER_PRIVATE3,&ctrl);
	  exit (0);
     }

     if (!strcmp(argv[argidx],"spdifin")) {
	  int val = atoi(argv[argidx+1]);
	  ioctl(fd,SOUND_MIXER_PRIVATE3,&ctrl);
	  printf("spdifin = %d\n",val);
	  ctrl.spdif_in = val;
	  ioctl(fdwr,SOUND_MIXER_PRIVATE3,&ctrl);
	  exit (0);
     }

     if (!strcmp(argv[argidx],"syncref")) {
	  int val = atoi(argv[argidx+1]);
	  ioctl(fd,SOUND_MIXER_PRIVATE3,&ctrl);
	  printf("syncref = %d\n",val);
	  ctrl.sync_ref = val;
	  ioctl(fdwr,SOUND_MIXER_PRIVATE3,&ctrl);
	  exit (0);
     }
 
     if (!strcmp(argv[argidx],"adat1cd")) {
	  int val = atoi(argv[argidx+1]);
	  ioctl(fd,SOUND_MIXER_PRIVATE3,&ctrl);
	  printf("adat1cd = %d\n",val);
	  ctrl.adat1_cd = val;
	  ioctl(fdwr,SOUND_MIXER_PRIVATE3,&ctrl);
	  exit (0);
     }
 
/* setting offset */

     if (argc < argidx+4)
	  usage();
	  
     if (!strcmp(argv[argidx],"offset")) {
	  rme_mixer mix;

	  mix.devnr = atoi(argv[argidx+1]);
	  
	  mix.i_offset = atoi(argv[argidx+2]);
	  mix.o_offset = atoi(argv[argidx+3]);
	  ioctl(fdwr,SOUND_MIXER_PRIVATE1,&mix);
	  fprintf(stdout,"devnr %d\n",mix.devnr);
	  fprintf(stdout,"mix.i_offset to %d\n",mix.i_offset);
	  fprintf(stdout,"mix.o_offset to %d\n",mix.o_offset);
	  exit (0);
     }	 

     usage();
     exit (0); /* to avoid warning */
}
